@@ -21,6 +21,10 @@ class Mcush( Instrument.SerialInstrument ):
     DEFAULT_NAME = 'MCUSH'
     DEFAULT_IDN = re.compile( 'mcush,([0-9]+\.[0-9]+.*)' )
     DEFAULT_MULTILINE_INPUT_LINE_LIMIT = 50
+    DEFAULT_CMD_LINE_LIMIT = 128
+    DEFAULT_CMD_ARGV_LIMIT = 20
+    DEFAULT_REBOOT_RETRY = 10
+    DEFAULT_DELAY_AFTER_REBOOT = 1
 
     regs_by_name = {}
     regs_by_addr = {}
@@ -339,6 +343,46 @@ class Mcush( Instrument.SerialInstrument ):
         ret = self.writeCommand('sys q')
         return ret
 
+    def reboot( self, delay=None ):
+        '''reboot controller'''
+        sync = False
+        retry = 0
+        try:
+            self.writeCommand( 'reboot' )
+            self.connect()
+            sync = True
+        except Instrument.ResponseError:
+            pass
+        except Instrument.CommandTimeoutError:
+            pass
+        while not sync: 
+            try:
+                self.connect()
+                sync = True
+            except:
+                retry = retry + 1
+                if retry > self.DEFAULT_REBOOT_RETRY:
+                    raise Instrument.CommandTimeoutError()
+        self.scpiIdn()
+        if delay is None:
+            time.sleep( self.DEFAULT_DELAY_AFTER_REBOOT )
+        else:
+            time.sleep( delay )
+
+
+    def getRebootCounter( self ):
+        cmd = 'reboot -c'
+        ret = self.writeCommand( cmd )
+        return int(ret[0])
+
+    def resetRebootCounter( self ):
+        cmd = 'reboot -r'
+        self.writeCommand( cmd )
+
+    def checkCommand( self, name ):
+        cmd = '? -c %s'% name
+        return bool(int(self.writeCommand(cmd)[0])) 
+
     def errnoStop( self ):
         '''stop errno blink'''
         self.writeCommand( 'error -s' )
@@ -430,7 +474,10 @@ class Mcush( Instrument.SerialInstrument ):
             self.setPrompts( self.DEFAULT_PROMPTS_MULTILINE )
             self.writeCommand( cmd )
             if b64:
-                buf = base64.encodestring(buf)
+                if Env.PYTHON_V3:
+                    buf = base64.encodebytes(buf).decode('utf8')
+                else:
+                    buf = base64.encodestring(buf)
             buf = buf.splitlines()
             for l in buf:
                 self.writeCommand( l.rstrip() )
@@ -560,6 +607,33 @@ class Mcush( Instrument.SerialInstrument ):
             self.writeCommand( line.rstrip() )
         self.setPrompts()
         self.writeCommand( '' )
+    
+    def fcfsFormat( self ):
+        self.writeCommand( 'fcfs -c format' )
+ 
+    def fcfsProgram( self, mem ):
+        # append zero to be 4-bytes alignment
+        while len(mem) & 0x3:
+            mem += Env.ZERO_BYTE
+        # prepare 32bit word list
+        words = [Utils.s2I(mem[4*i:4*(i+1)]) for i in range(int(len(mem)/4))]
+        offset = 0
+        while words:
+            cmd = 'fcfs -c program'
+            if offset:
+                cmd += ' -o %d'% (offset*4)
+            count_argv = len(cmd.split(' '))
+            while words:
+                wr = ' %d'% words[0] 
+                if len(cmd + wr) > self.DEFAULT_CMD_LINE_LIMIT:
+                    break
+                words.pop(0)
+                cmd += wr
+                offset += 1
+                count_argv += 1
+                if count_argv >= self.DEFAULT_CMD_ARGV_LIMIT:
+                    break
+            self.writeCommand( cmd )
 
     def sgpioStop( self ):
         self.writeCommand( 'sgpio --stop' )
@@ -949,6 +1023,22 @@ class Mcush( Instrument.SerialInstrument ):
             ret.append( (id, ext, rtr, binascii.unhexlify(dat)) )
         return ret
 
+    def canFilterReset( self, add_default=True ):
+        # format: <filter_id> <enabled>  <can_id> <mask>
+        # eg: can -c filter 5 0            (disable #5)
+        # eg: can -c filter 0 1 0x01 0x1F  (enable #0 as 0x1/0x1F)
+        # eg: can -e -r -c filter 0 1 0x01 0x1F  (enable #0 as 0x1/0x1F ext/rtr set)
+        for line in self.can( 'filter' ):
+            index = int(line.split(': ')[0].lstrip('#'))
+            self.can( 'filter', args=[index, 0] )
+        if add_default:
+            # add two filters to receive all std/ext frames
+            self.can( 'filter', args=[0, 1, 0, 0] )
+            self.can( 'filter', args=[1, 1, 0, 0], ext=True )
+
+    def canFilterSet( self, index, enable, can_id, mask, ext=None, rtr=None ):
+        self.can( 'filter', ext=ext, rtr=rtr, args=[index, int(enable), can_id, mask] )
+ 
     def env( self ):
         return Utils.parseKeyValueLines( self.writeCommand('env') )
 
