@@ -3,7 +3,7 @@ __doc__ = 'Shell Lab Series'
 __author__ = 'Peng Shulin <trees_peng@163.com>'
 __license__ = 'MCUSH designed by Peng Shulin, all rights reserved.'
 from re import compile as re_compile
-from .. import Mcush, Instrument, Utils
+from .. import Mcush, Instrument, Utils, Env
 import binascii
 import time
 
@@ -110,12 +110,12 @@ class ShellLab(Mcush.Mcush):
 
 COLOR_TAB = {
 'black'  : 0x000000,
-'red'    : 0x0000FF,
+'red'    : 0xFF0000,
 'green'  : 0x00FF00,
-'yellow' : 0x00FFFF,
-'blue'   : 0xFF0000,
+'blue'   : 0x0000FF,
+'yellow' : 0xFFFF00,
+'cyan'   : 0x00FFFF,
 'purple' : 0xFF00FF,
-'cyan'   : 0xFFFF00,
 'white'  : 0xFFFFFF,
 }
 
@@ -135,7 +135,7 @@ class ShellLabLamp(Mcush.Mcush):
         if blue is not None:
             cmd += ' -b %d'% blue
         if freq is not None:
-            cmd += ' -f %d'% freq
+            cmd += ' -f %s'% freq
         return self.writeCommand(cmd)
 
     def alarm( self, count=None, freq=None ):
@@ -152,7 +152,7 @@ class ShellLabLamp(Mcush.Mcush):
 
     def color( self, c, freq=None ):
         if isinstance(c, str):
-            if not COLOR_TAB.has_key(c):
+            if not c in COLOR_TAB:
                 raise Exception('Unknown color name %s'% c)
             c = COLOR_TAB[c]
         self.lamp( c, freq=freq ) 
@@ -163,16 +163,15 @@ class ShellLabStrap(Mcush.Mcush):
     DEFAULT_IDN = re_compile( 'ShellLab-L2[a-zA-Z]*,([0-9]+\.[0-9]+.*)' )
 
     def __init__( self, *args, **kwargs ):
-        if not 'length' in kwargs:
-            raise Exception('length not assigned')
         Instrument.Instrument.__init__( self, *args, **kwargs )
-        self.length = kwargs['length']
-        self.strapLength( self.length )
+        self.length = kwargs.get('length', None)
+        if self.length is not None:
+            self.strapLength( self.length ) 
 
     def strapLength( self, length ):
         cmd = 'strap -l%d'% length
         self.writeCommand(cmd)
- 
+
     def strap( self, color=None, red=None, green=None, blue=None, freq=None ):
         cmd = 'strap'
         if color is not None:
@@ -184,15 +183,15 @@ class ShellLabStrap(Mcush.Mcush):
         if blue is not None:
             cmd += ' -b %d'% blue
         if freq is not None:
-            cmd += ' -f %d'% freq
+            cmd += ' -f %s'% freq
         return self.writeCommand(cmd)
 
-    def reset( self, lamp_freq=1 ):
-        self.strap( 0, freq=lamp_freq )
+    def reset( self, freq=1 ):
+        self.strap( 0, freq=freq )
 
     def color( self, c, freq=None ):
         if isinstance(c, str):
-            if not COLOR_TAB.has_key(c):
+            if not c in COLOR_TAB:
                 raise Exception('Unknown color name %s'% c)
             c = COLOR_TAB[c]
         self.strap( c, freq=freq ) 
@@ -206,7 +205,7 @@ class ShellLabStaticStrap(ShellLabStrap):
     def __init__( self, *args, **kwargs ):
         ShellLabStrap.__init__(self, *args, **kwargs)
         setting = Utils.parseKeyValueLines(self.strap())
-        if int(setting['freq']) != 0:
+        if float(setting['freq']) > 0:
             self.strap( freq=0 )
         self._color = int(setting['color'], 16)
         self._brightness = kwargs.get('brightness', 1.0)
@@ -401,7 +400,7 @@ class ShellLabCANopen(ShellLabCAN):
                         responsed = True
                         break
             control ^= 0x10
-        return ''.join(read)
+        return Env.EMPTY_BYTE.join(read)
      
     def writeObject( self, id, index, subindex, val ):
         val_len = len(val)
@@ -410,7 +409,10 @@ class ShellLabCANopen(ShellLabCAN):
         if val_len <= 4:
             # expedited mode
             control |= 0x02
-            v = map(ord, val)
+            if Env.PYTHON_V3:
+                v = list(val)
+            else:
+                v = map(ord, val)
             if len(v) < 4:
                 control |= 0x01  # indicated 
                 control |= (4-len(v)) << 2 # not used bytes
@@ -443,7 +445,10 @@ class ShellLabCANopen(ShellLabCAN):
         # remaining segment request
         control = 0x00
         while bytes_written < val_len:
-            v = map(ord, val[bytes_written:bytes_written+7])
+            if Env.PYTHON_V3:
+                v = list(val[bytes_written:bytes_written+7])
+            else:
+                v = map(ord, val[bytes_written:bytes_written+7])
             bytes_written += len(v)
             if bytes_written >= val_len:
                 control |= 0x01  # no more flag
